(function() {
  var cnh, colors, cp, dirWatcherFactory, echoUsageAndExit, fs, path, runTestFile, util;
  cnh = require('./coffee-new-helper');
  colors = require('colors');
  cp = require('child_process');
  dirWatcherFactory = require('dir-watcher');
  fs = require('fs');
  path = require('path');
  util = require('util');
  echoUsageAndExit = function() {
    console.warn('Usage: coffee-new <type> <app name> <author> <description>');
    console.warn('<type> can be "cli" or "library"');
    return process.exit(-1);
  };
  runTestFile = function(file) {
    util.log('Running test file '.yellow + file.yellow.bold);
    return cnh.run('expresso', [file]);
  };
  exports.autoTest = function(persist) {
    var cwd, srcWatcher, testWatcher;
    if (persist == null) {
      persist = true;
    }
    util.log('Watching...');
    cwd = process.cwd();
    dirWatcherFactory.setup(true);
    srcWatcher = dirWatcherFactory.create(function(entry) {
      var testFile;
      if (path.extname(entry) === '.coffee') {
        testFile = cnh.getMirrorPath(cwd, entry, 'src', 'test');
        testFile = cnh.changeFileExtension(testFile, '.coffee', '-test.coffee');
        return path.exists(testFile, function(exists) {
          var relativeTestPath;
          relativeTestPath = cnh.getRelativePath(cwd, testFile);
          if (exists) {
            return runTestFile(relativeTestPath);
          } else {
            return util.log(('Test file ' + relativeTestPath(" doesn't exist.")).red);
          }
        });
      }
    });
    srcWatcher.watch('src');
    testWatcher = dirWatcherFactory.create(function(entry) {
      var relativeTestPath;
      if (path.extname(entry) === '.coffee') {
        relativeTestPath = cnh.getRelativePath(cwd, entry);
        return runTestFile(relativeTestPath);
      }
    });
    return testWatcher.watch('test');
  };
  exports.build = function(callback) {
    util.log('Building...');
    return cnh.run('coffee', ['-c', '-o', 'lib/', 'src/'], function() {
      util.log('Building complete.');
      if (callback != null) {
        return callback();
      }
    });
  };
  exports.cleanLibDirectory = function(callback) {
    return cp.exec('rm -rf lib/*', function(err, stdout, stderr) {
      if (err) {
        console.warn(stderr);
      }
      if (callback != null) {
        return callback();
      }
    });
  };
  exports.run = cnh.run;
  exports.validateRootAccess = function() {
    if (process.env['USER'] !== 'root') {
      console.log('Must be run as root');
      return process.exit(-1);
    }
  };
  exports.start = function() {
    var additionalPackageCode, appAuthor, appCode, appDescription, appName, appType;
    if (process.argv.length !== 6) {
      console.warn('Error: Incorrect number of parameters');
      echoUsageAndExit();
    }
    appType = process.argv[2];
    appName = process.argv[3];
    appAuthor = process.argv[4];
    appDescription = process.argv[5];
    if (appType !== 'cli' && appType !== 'library') {
      console.warn('Error: First parameter <type> is incorrect');
      echoUsageAndExit();
    }
    if (path.existsSync("" + appName + "/")) {
      console.warn("Error: Directory " + appName + " already exists");
      process.exit(-1);
    }
    if (appType === 'cli') {
      appCode = "# requires\nprogram = require 'commander'\n\n# globals\n\n# private functions\n\n# public functions\nexports.start = () ->\n	program\n		.version('0.0.1')\n		.option('-o, --option <option>', 'option description here')\n\n	program\n		.command('doCommand <arg1> <arg2>')\n		.description('command description here')\n		.action (arg1, arg2) ->\n			# code goes here\n	\n	program.parse process.argv\n	# if required, you can also parse program.args";
      additionalPackageCode = ",\n  \"dependencies\": {\n    \"colors\": \">=0.5.0\",\n    \"commander\": \">=0.1.0\"\n  },\n  \"bin\": {\n    \"" + appName + "\": \"./bin/" + appName + "\"\n  }";
    } else if (appType === 'library') {
      appCode = "# requires\n\n# globals\n\n# private functions\n\n# public functions\nexports.doSomething = ->\n	# code goes here";
      additionalPackageCode = ",\n  \"dependencies\": {\n  }";
    } else {
      throw 'appType ' + appType + " doesn't exist.";
    }
    fs.mkdirSync("" + appName + "/", 0744);
    fs.writeFile("" + appName + "/.gitignore", "lib/*");
    fs.writeFile("" + appName + "/.npmignore", ".git*\nCakefile\nnpm-debug.log\nsrc/\ntest/\nREADME.md");
    if (appType === 'cli') {
      fs.mkdir("" + appName + "/bin/", 0744, function() {
        return fs.writeFile("" + appName + "/bin/" + appName, "#!/usr/bin/env node\nrequire('../lib/" + appName + "').start();", function(err) {
          return fs.chmod("" + appName + "/bin/" + appName, 0755);
        });
      });
    }
    fs.writeFile("" + appName + "/Cakefile", "cn = require 'coffee-new'\n\ntask 'autotest', 'watch & test when changes are detected', (options) ->\n	cn.autoTest()\n\ntask 'build', 'compile all files in src/ to be placed in lib/', (options) ->\n	cn.build()\n\ntask 'clean', 'delete everything in lib/', (options) ->\n	cn.cleanLibDirectory()\n\ntask 'install', 'build & run npm -g install', (options) ->\n	cn.validateRootAccess()\n	cn.build -> cn.run 'npm', ['-g', 'install']\n\ntask 'test', 'run the entire test suite', (options) ->\n	cn.run 'expresso'\n\ntask 'uninstall', 'run npm -g uninstall', (options) ->\n	cn.validateRootAccess()\n	cn.run 'npm', ['-g', 'uninstall', '" + appName + "']");
    fs.mkdir("" + appName + "/lib/", 0744);
    fs.mkdir("" + appName + "/src/", 0744, function() {
      return fs.writeFile("" + appName + "/src/" + appName + ".coffee", appCode);
    });
    fs.mkdir("" + appName + "/test/", 0744, function() {
      return fs.writeFile("" + appName + "/test/" + appName + "-test.coffee", "app = require '../src/" + appName + "'\nshould = require 'should'\n\nexports.testTodo = ->\n	1.should.be.equal 1");
    });
    fs.writeFile("" + appName + "/README.md", "# " + appName + "\n" + appDescription + "\n\nEnjoy!\n\n" + appAuthor);
    return fs.writeFile("" + appName + "/package.json", "{\n  \"name\": \"" + appName + "\",\n  \"description\": \"" + appDescription + "\",\n  \"author\": \"" + appAuthor + "\",\n  \"version\": \"0.0.1\",\n  \"main\": \"lib/" + appName + "\",\n  \"licenses\": [{\n    \"type\": \"MIT\",\n    \"url\": \"http://opensource.org/licenses/mit-license.php\"\n  }],\n  \"engines\": {\n    \"node\": \">=0.4.5\"\n  }" + additionalPackageCode + "\n}");
  };
}).call(this);
